class ScenarioCompilerError(Exception):
    """Base exception for :class:`ScenarioCompiler` errors."""


class IncorrectScenarioTextError(ScenarioCompilerError):
    """Exception for incorrect scenario text format."""


class ScenarioParserError(Exception):
    """Base exception for parsing error."""


class FeatureNameParsingError(ScenarioParserError):
    """Exception for feature name parsing error."""


class FeatureTypeParsingError(ScenarioParserError):
    """Exception for feature type parsing error."""


class AdditionalInfoParsingError(ScenarioParserError):
    """Exception for additional info parsing error."""


class DatetimeParsingError(ScenarioParserError):
    """Exception for datetime parsing error."""



# flake8: noqa
from .manager import (
    DefaultAdminAuthorizationManager,
    IAdminAuthorizationManager,
    LDAPAdminAuthorizationManager,
    LDAPAuthenticator,
    SimpleAdminAuthorizationManager,
)
from .settings import AuthorizationStrategy, OverhaveAuthorizationSettings, OverhaveLdapClientSettings
from .strategies import AuthorizationStrategy



from typing import Optional

from pydantic.datetime_parse import timedelta

from overhave.authorization.strategies import AuthorizationStrategy
from overhave.base_settings import BaseOverhavePrefix


class OverhaveAuthorizationSettings(BaseOverhavePrefix):
    """Settings for Overhave authorization in components interface.

    Supports 3 strategies: SIMPLE, DEFAULT and LDAP.
    LDAP authorization uses group politics with administration group `admin_group`.
    SIMPLE and DEFAULT strategies use admin user that would be dynamically created at startup.
    """

    auth_strategy: AuthorizationStrategy = AuthorizationStrategy.SIMPLE
    admin_group: Optional[str] = None


class OverhaveLdapClientSettings(BaseOverhavePrefix):
    """Settings for Overhave LDAP client for AuthorizationStrategy.LDAP strategy."""

    ldap_url: str  # for example: "ldap://mydomain.ru"
    ldap_domain: str  # for example: "domain\\"
    ldap_dn: str  # for example: "dc=example,dc=com"
    ldap_timeout: timedelta = timedelta(seconds=10)



from pydantic.env_settings import BaseSettings

from overhave.base_settings import OVERHAVE_ENV_PREFIX


class OverhaveApiSettings(BaseSettings):
    """Settings for Overhave API server, started with Uvicorn."""

    host: str = "0.0.0.0"
    port: int = 8000
    workers: int = 1

    class Config:
        env_prefix = OVERHAVE_ENV_PREFIX + "UVICORN_"



import fastapi as fastapi

from overhave.api.views import test_user_handler


def create_overhave_api() -> fastapi.FastAPI:
    test_user_router = fastapi.APIRouter()
    test_user_router.add_api_route("/", test_user_handler, methods=["GET"])

    app = fastapi.FastAPI()
    app.include_router(test_user_router, prefix="/test_user", tags=["test_users"])
    return app



import abc
from typing import Optional, cast

from overhave import db
from overhave.entities import TestRunModel
from overhave.utils import get_current_time


class ITestRunStorage(abc.ABC):
    """Abstract class for test runs storage."""

    @abc.abstractmethod
    def create_test_run(self, scenario_id: int, executed_by: str) -> int:
        pass

    @abc.abstractmethod
    def set_run_status(self, run_id: int, status: db.TestRunStatus, traceback: Optional[str] = None) -> None:
        pass

    @abc.abstractmethod
    def set_report(self, run_id: int, status: db.TestReportStatus, report: Optional[str] = None) -> None:
        pass

    @abc.abstractmethod
    def get_test_run(self, run_id: int) -> Optional[TestRunModel]:
        pass


class TestRunStorage(ITestRunStorage):
    """Class for test runs storage."""

    def create_test_run(self, scenario_id: int, executed_by: str) -> int:
        with db.create_session() as session:
            scenario: db.Scenario = session.query(db.Scenario).filter(db.Scenario.id == scenario_id).one()
            feature: db.Feature = session.query(db.Feature).filter(db.Feature.id == scenario.feature_id).one()
            run = db.TestRun(  # type: ignore
                scenario_id=scenario_id,
                name=feature.name,
                status=db.TestRunStatus.STARTED,
                report_status=db.TestReportStatus.EMPTY,
                executed_by=executed_by,
            )
            session.add(run)
            session.flush()
            return cast(int, run.id)

    def set_run_status(self, run_id: int, status: db.TestRunStatus, traceback: Optional[str] = None) -> None:
        with db.create_session() as session:
            run: db.TestRun = session.query(db.TestRun).filter(db.TestRun.id == run_id).one()
            run.status = status
            if status is db.TestRunStatus.RUNNING:
                run.start = get_current_time()
            if status.finished:
                run.end = get_current_time()
            if isinstance(traceback, str):
                run.traceback = traceback

    def set_report(self, run_id: int, status: db.TestReportStatus, report: Optional[str] = None) -> None:
        with db.create_session() as session:
            run: db.TestRun = session.query(db.TestRun).filter(db.TestRun.id == run_id).one()
            run.report_status = status
            if isinstance(report, str):
                run.report = report

    def get_test_run(self, run_id: int) -> Optional[TestRunModel]:
        with db.create_session() as session:
            run: Optional[db.TestRun] = session.query(db.TestRun).get(run_id)
            if run is not None:
                return cast(TestRunModel, TestRunModel.from_orm(run))
            return None



from typing import Any

import flask
from flask import redirect, request, url_for
from flask_login import current_user
from wtforms import PasswordField

from overhave import db
from overhave.admin.views.base import ModelViewConfigured


class AccessModelView(ModelViewConfigured):
    """Additional base_factory view for access views."""

    can_view_details = False

    form_overrides = dict(password=PasswordField)

    def is_accessible(self) -> bool:
        return bool(current_user.is_authenticated and current_user.role == db.Role.admin)

    def inaccessible_callback(self, name: str, **kwargs: Any) -> Any:
        # redirect to login page if user doesn't have access
        if current_user.is_authenticated:
            flask.flash("You have not got access for users and groups management!", category="error")
            return redirect(url_for("admin.index"))
        return redirect(url_for("admin.login", next=request.url))


class UserView(AccessModelView):
    """View for application user access management."""

    column_list = ["id", "login", "role", "created_at"]
    column_searchable_list = ["login"]


class GroupView(AccessModelView):
    """View for application group access management."""

    column_searchable_list = ["group"]



from yarl import URL

from overhave.transport.http.base_client import BaseHttpClientSettings


class OverhaveStashClientSettings(BaseHttpClientSettings):
    """Settings for :class:`StashHttpClient`."""

    pr_path: str = "rest/api/1.0/projects/{project_key}/repos/{repository_name}/pull-requests"
    auth_token: str

    class Config:
        env_prefix = "OVERHAVE_STASH_"

    def get_pr_url(self, project_key: str, repository_name: str) -> URL:
        return self.url / self.pr_path.format(project_key=project_key, repository_name=repository_name)



# flake8: noqa
from .common import add_scenario_title_to_report, get_description_manager, get_scenario, get_step_context_runner
from .description_manager import DescriptionManager
from .step_context_runner import StepContextNotDefinedError, StepContextRunner



from typing import List

from overhave.entities import FeatureTypeName
from overhave.publication.settings import BaseGitPublisherSettings
from overhave.transport.http.gitlab_client import GitlabRepository


class NotSpecifiedFeatureTypeError(RuntimeError):
    """Exception for not specified reviewers relative to feature type."""


class OverhaveGitlabPublisherSettings(BaseGitPublisherSettings):
    """Settings for :class:`GitlabVersionPublisher`.

    This is a representation of Gitlab project parameters.
    Some merge-request parameters are also could be defined through these settings.
    """

    repository_id: str  # for example '2034'

    class Config:
        env_prefix = "OVERHAVE_GITLAB_"

    @property
    def repository(self) -> GitlabRepository:
        return GitlabRepository(project_id=self.repository_id)

    @property
    def target_branch(self) -> str:
        return self.default_target_branch_name

    def get_reviewers(self, feature_type: FeatureTypeName) -> List[str]:
        if self.feature_type_to_reviewers_mapping:
            reviewers = self.feature_type_to_reviewers_mapping.get(feature_type)
            if not reviewers:
                raise NotSpecifiedFeatureTypeError(
                    f"'{feature_type}' reviewers are not specified in 'feature_type_to_reviewers_mapping' dict!"
                )
            return reviewers
        return self.default_reviewers



import logging
from typing import Optional

from pydantic import SecretStr

from overhave.authorization.manager.secret_mixin import AdminSecretMixin
from overhave.entities import SystemUserModel

logger = logging.getLogger(__name__)


class BaseSimpleAdminAuthorizationManagerException(Exception):
    """Base exception for :class:`SimpleAdminAuthorizationManager`."""


class NullablePasswordError(Exception):
    """Exception for situation when row is without password."""


class SimpleAdminAuthorizationManager(AdminSecretMixin):
    """Class for user registration.

    Manager does not provide real authorization. Each user could use preferred name.
    This name will be used for user authority. Each user is unique. Passwords not required.
    """

    def authorize_user(self, username: str, password: SecretStr) -> Optional[SystemUserModel]:
        user = self._system_user_storage.get_user_by_credits(login=username)
        if user is None:
            user = self._system_user_storage.create_user(login=username, password=password)
        if user.password is None:
            raise NullablePasswordError(f"User with id={user.id} has not got password!")
        if user.password.get_secret_value() == password.get_secret_value():
            return user
        return None



from datetime import datetime
from typing import List, NewType, Optional

from pydantic.main import BaseModel
from pydantic.types import SecretStr
from pydantic_sqlalchemy import sqlalchemy_to_pydantic

from overhave.db import (
    Draft,
    DraftStatus,
    Emulation,
    EmulationRun,
    Feature,
    FeatureType,
    Role,
    Scenario,
    Tags,
    TestReportStatus,
    TestRun,
    TestRunStatus,
    TestUser,
    UserRole,
)
from overhave.entities.feature import FeatureTypeName

TestUserSpecification = NewType("TestUserSpecification", dict[str, str])


class SystemUserModel(sqlalchemy_to_pydantic(UserRole)):  # type: ignore
    """Model for :class:`UserRole`."""

    id: int
    login: str
    password: Optional[SecretStr]
    role: Role

    @property
    def is_authenticated(self) -> bool:
        return True

    @property
    def is_active(self) -> bool:
        return True

    @property
    def is_anonymous(self) -> bool:
        return False

    def get_id(self) -> int:
        return self.id

    def __unicode__(self) -> str:
        return self.login


class FeatureTypeModel(sqlalchemy_to_pydantic(FeatureType)):  # type: ignore
    """Model for :class:`FeatureType` row."""

    id: int
    name: FeatureTypeName


class TagModel(sqlalchemy_to_pydantic(Tags)):  # type: ignore
    """Model for :class:`Tags` row."""

    id: int
    value: str
    created_by: str


class FeatureModel(sqlalchemy_to_pydantic(Feature)):  # type: ignore
    """Model for :class:`Feature` row."""

    id: int
    name: str
    author: str
    type_id: int
    last_edited_by: str
    last_edited_at: datetime
    task: List[str]
    file_path: str
    released: bool

    feature_type: FeatureTypeModel
    feature_tags: List[TagModel]


class ScenarioModel(sqlalchemy_to_pydantic(Scenario)):  # type: ignore
    """Model for :class:`Scenario` row."""

    id: int
    text: str
    feature_id: int


class TestRunModel(sqlalchemy_to_pydantic(TestRun)):  # type: ignore
    """Model for :class:`TestRun` row."""

    __test__ = False

    id: int
    created_at: datetime
    name: str
    executed_by: str
    start: Optional[datetime]
    end: Optional[datetime]
    status: TestRunStatus
    report_status: TestReportStatus
    report: Optional[str]
    traceback: Optional[str]
    scenario_id: int


class DraftModel(sqlalchemy_to_pydantic(Draft)):  # type: ignore
    """Model for :class:`Draft` row."""

    id: int
    feature_id: int
    test_run_id: int
    pr_url: Optional[str]
    published_by: str
    published_at: Optional[datetime]
    traceback: Optional[str]
    status: DraftStatus


class TestExecutorContext(BaseModel):
    """Context model for test execution."""

    __test__ = False

    feature: FeatureModel
    scenario: ScenarioModel
    test_run: TestRunModel


class PublisherContext(TestExecutorContext):
    """Context model for version publishing."""

    draft: DraftModel
    target_branch: str


class TestUserModel(sqlalchemy_to_pydantic(TestUser)):  # type: ignore
    """Model for :class:`TestUser` row."""

    __test__ = False

    id: int
    name: str
    created_by: str
    specification: TestUserSpecification
    feature_type_id: int
    feature_type: FeatureTypeModel


class EmulationModel(sqlalchemy_to_pydantic(Emulation)):  # type: ignore
    """Model for :class:`Emulation` row."""

    test_user: TestUserModel


class EmulationRunModel(sqlalchemy_to_pydantic(EmulationRun)):  # type: ignore
    """Model for :class:`EmulationRun` row."""

    emulation: EmulationModel



import logging
from datetime import datetime
from pathlib import Path
from typing import List

import pytz

from overhave.entities import (
    BaseFileExtractor,
    FeatureExtractor,
    FeatureModel,
    GitRepositoryInitializer,
    OverhaveFileSettings,
    ScenarioModel,
    TagModel,
)
from overhave.scenario import ScenarioParser
from overhave.scenario.parser import FeatureInfo
from overhave.storage import (
    IDraftStorage,
    IFeatureStorage,
    IFeatureTagStorage,
    IFeatureTypeStorage,
    IScenarioStorage,
    ISystemUserStorage,
)
from overhave.synchronization.abstract import IOverhaveSynchronizer

logger = logging.getLogger(__name__)


class BaseOverhaveSynchronizerException(Exception):
    """Base exception for :class:`OverhaveSynchronizer`."""


class NullableLastEditedAtError(BaseOverhaveSynchronizerException):
    """Exception for nullable last_edited_at."""


class NullableInfoLastEditedByError(BaseOverhaveSynchronizerException):
    """Exception for nullable feature info last_edited_at."""


class NullableInfoNameError(BaseOverhaveSynchronizerException):
    """Exception for situation without feature info name."""


class NullableInfoScenariosError(BaseOverhaveSynchronizerException):
    """Exception for situation without feature info scenarios."""


class NullableInfoAuthorError(BaseOverhaveSynchronizerException):
    """Exception for situation without feature info author."""


class NullableInfoFeatureTypeError(BaseOverhaveSynchronizerException):
    """Exception for situation without feature info type."""


class FeatureInfoUserNotFoundError(BaseOverhaveSynchronizerException):
    """Exception for situation without specified user in database."""


class OverhaveSynchronizer(BaseFileExtractor, IOverhaveSynchronizer):
    """Class for synchronization between git and database."""

    def __init__(
        self,
        file_settings: OverhaveFileSettings,
        feature_storage: IFeatureStorage,
        feature_type_storage: IFeatureTypeStorage,
        scenario_storage: IScenarioStorage,
        draft_storage: IDraftStorage,
        tag_storage: IFeatureTagStorage,
        scenario_parser: ScenarioParser,
        feature_extractor: FeatureExtractor,
        system_user_storage: ISystemUserStorage,
        git_initializer: GitRepositoryInitializer,
    ):
        super().__init__(extenstion=file_settings.feature_suffix)
        self._file_settings = file_settings
        self._feature_storage = feature_storage
        self._feature_type_storage = feature_type_storage
        self._scenario_storage = scenario_storage
        self._draft_storage = draft_storage
        self._tag_storage = tag_storage
        self._scenario_parser = scenario_parser
        self._feature_extractor = feature_extractor
        self._system_user_storage = system_user_storage
        self._git_initializer = git_initializer

    @staticmethod
    def _update_feature_model_with_info(
        model: FeatureModel, info: FeatureInfo, file_ts: datetime, tags: List[TagModel]
    ) -> None:
        if info.name is None:
            raise NullableInfoNameError("Feature info has not got feature name!")
        model.name = info.name
        if info.last_edited_by is not None:
            model.last_edited_by = info.last_edited_by
        model.last_edited_at = file_ts
        if info.tasks is not None:
            model.task = info.tasks
        model.feature_tags = tags

    def _get_last_change_time(self, model: FeatureModel) -> datetime:
        draft_models = self._draft_storage.get_drafts_by_feature_id(model.id)
        if draft_models:
            logger.info("Feature has got drafts.")
            last_published_draft = draft_models[-1]
            if last_published_draft.published_at is not None:
                logger.info(
                    "Last version has been published at %s.",
                    last_published_draft.published_at.strftime("%d-%m-%Y %H:%M:%S"),
                )
                return last_published_draft.published_at
        logger.info("Feature hasn't got any published version.")
        return model.last_edited_at

    def _get_feature_tags(self, info: FeatureInfo) -> List[TagModel]:
        tags: List[TagModel] = []
        if info.tags is not None:
            if info.last_edited_by is None:
                raise NullableInfoLastEditedByError("last_edited_by value should not be None!")
            for tag in info.tags:
                tag_model = self._tag_storage.get_or_create_tag(value=tag, created_by=info.last_edited_by)
                tags.append(tag_model)
        return tags

    def _update_db_feature(self, model: FeatureModel, info: FeatureInfo, file_ts: datetime) -> None:
        logger.info("Feature is gonna be updated...")
        if info.scenarios is None:
            raise NullableInfoScenariosError("Some troubles with parsing feature - could not get scenarios!")
        feature_tags = self._get_feature_tags(info=info)
        self._update_feature_model_with_info(model=model, info=info, file_ts=file_ts, tags=feature_tags)
        scenario_model = self._scenario_storage.get_scenario_by_feature_id(model.id)
        self._feature_storage.update_feature(model)
        scenario_model.text = info.scenarios
        self._scenario_storage.update_scenario(model=scenario_model)
        logger.info("Feature has been updated successfully.")

    def _create_feature(self, file: Path, info: FeatureInfo) -> None:
        logger.info("Feature is gonna be created...")
        if info.name is None:
            raise NullableInfoNameError("Feature info has not got feature name!")
        if info.type is None:
            raise NullableInfoFeatureTypeError("Could not create feature without feature type!")
        if info.author is None:
            raise NullableInfoAuthorError("Could not create feature without author!")
        for user in (info.author, info.last_edited_by):
            if self._system_user_storage.get_user_by_credits(login=info.author) is not None:
                continue
            raise FeatureInfoUserNotFoundError(f"Could not find user with login={user}!")
        feature_tags = self._get_feature_tags(info=info)
        feature_type = self._feature_type_storage.get_feature_type_by_name(info.type)
        if info.last_edited_by is None:
            info.last_edited_by = info.author
        feature_model = FeatureModel(
            id=0,
            name=info.name,
            author=info.author,
            type_id=feature_type.id,
            last_edited_by=info.last_edited_by,
            last_edited_at=info.last_edited_at,
            task=info.tasks or [],
            file_path=file.relative_to(
                self._feature_extractor.feature_type_to_dir_mapping[feature_type.name]
            ).as_posix(),
            released=True,
            feature_type=feature_type,
            feature_tags=feature_tags,
        )
        feature_model.id = self._feature_storage.create_feature(feature_model)
        scenario_model = ScenarioModel(id=0, feature_id=feature_model.id, text=info.scenarios)
        self._scenario_storage.create_scenario(scenario_model)
        logger.info("Feature with ID=%s has been created successfully.", feature_model.id)

    def synchronize(self, create_db_features: bool = False, pull_repository: bool = False) -> None:  # noqa: C901
        if pull_repository:
            self._git_initializer.pull()
        logger.info("Start synchronization...")
        all_features = self._extract_recursively(self._file_settings.features_dir)
        for feature_file in all_features:
            logger.info("Synchronize feature from file %s...", feature_file.as_posix())
            feature_info = self._scenario_parser.parse(feature_file.read_text())
            logger.debug("Parsed feature info: %s", feature_info)
            if feature_info.id is None:
                if not create_db_features:
                    logger.warning("Feature doesn't have Overhave ID or ID format is incorrect. Skip.")
                    continue
                self._create_feature(file=feature_file, info=feature_info)
                continue
            logger.info("Feature has ID=%s", feature_info.id)
            feature_model = self._feature_storage.get_feature(feature_info.id)
            if feature_model is None:
                logger.warning("Feature doesn't exist in Overhave database.")
                continue  # TODO: unlink file and create MR with deletions at the end

            feature_file_ts = datetime.fromtimestamp(feature_file.stat().st_mtime, tz=pytz.UTC)
            if feature_model.last_edited_at is None:
                raise NullableLastEditedAtError("last_edited_at value should not be None!")
            if feature_model.last_edited_at == feature_file_ts and feature_model.released:
                logger.warning("Feature has been already synchronized.")
                continue

            last_change_time = self._get_last_change_time(model=feature_model)
            if last_change_time < feature_file_ts:
                self._update_db_feature(model=feature_model, info=feature_info, file_ts=feature_file_ts)
                continue
            if feature_model.released:
                logger.info("Feature is already actual. Skip.")
                continue
            logger.info(
                "Feature was changed soon (at %s), but not released. Skip.",
                last_change_time.strftime("%d-%m-%Y %H:%M:%S"),
            )
        logger.info("Synchronization completed.")



from typing import Any, Dict, List, Mapping, Optional, Type

from pydantic import BaseModel, validator
from yarl import URL

from overhave.base_settings import BaseOverhavePrefix
from overhave.utils import make_url


class EmptyBrowseURLError(ValueError):
    """Exception for situation with empty ```browse_url``` while trying to ```get_link_url```."""


class OverhaveProjectSettings(BaseOverhavePrefix):
    """
    Entity project settings.

    You could specify your fixture content for specific pytest usage with your own plugins.
    It could give you pretty extended file with powerful possibilities.
    `fixture_context` will be formatted by `OverhaveProccessor` before test would be started.
    """

    # Fixture content in list format, which would be compiled into formatted string.
    fixture_content: List[str] = [
        "from pytest_bdd import scenarios",
        "from overhave import overhave_proxy_manager",
        "pytest_plugins = overhave_proxy_manager().plugin_resolver.get_plugins()",
        "scenarios('{feature_file_path}')",
    ]

    # Task tracker URL for browsing
    browse_url: Optional[URL]
    # Behaviour specification keyword for tasks attachment and linking
    links_keyword: Optional[str]

    # Templates are used for creation of test user and system specifications.
    # Keys for specification mapping are still the same - feature types.
    user_spec_template_mapping: Mapping[str, Type[BaseModel]] = {}

    @validator("browse_url", pre=True)
    def make_browse_url(cls, v: Optional[str]) -> Optional[URL]:
        return make_url(v)

    @validator("links_keyword")
    def validate_links_keyword(cls, v: Optional[str], values: Dict[str, Any]) -> Optional[str]:
        if isinstance(v, str) and values.get("browse_url") is None:
            raise ValueError("Browse URL should be specified in case of links keyword usage!")
        return v

    def get_link_url(self, link: str) -> str:
        if isinstance(self.browse_url, URL):
            return (self.browse_url / link).human_repr()
        raise EmptyBrowseURLError("Browse URL is None, so could not create link URL!")


class OverhaveTestSettings(BaseOverhavePrefix):
    """Settings for PytestRunner, which runs scenario tests with specified addoptions."""

    default_pytest_addoptions: str = "--disable-warnings"
    extra_pytest_addoptions: Optional[str]

    workers: Optional[int]  # Number of xdist workers, `None` by default



from datetime import datetime
from typing import Final, List, Optional

from pydantic import BaseModel, Field


class GitlabRepository(BaseModel):
    """Model for Gitlab merge-request repository."""

    id: str = Field(..., alias="project_id")


class GitlabMrRequest(BaseModel):
    """Model for Gitlab merge-request request."""

    id: str = Field(..., alias="project_id")
    source_branch: str
    target_branch: str
    title: Optional[str]
    description: Optional[str]
    reviewer_ids: List[str]


class GitlabMrCreationResponse(BaseModel):
    """Model for Gitlab merge-request creation response."""

    created_at: datetime
    updated_at: datetime
    web_url: Optional[str]
    traceback: Optional[Exception]
    state: str

    class Config:
        arbitrary_types_allowed = True

    @property
    def get_mr_url(self) -> str:
        if isinstance(self.web_url, str):
            return self.web_url
        raise RuntimeError("Could not get merge-request URL from response!")


AnyGitlabResponseModel = GitlabMrCreationResponse
GITLAB_RESPONSE_MODELS: Final = [GitlabMrCreationResponse]



import typer

from overhave.base_settings import DataBaseSettings
from overhave.cli.db_cmds.regular import create_schema, drop_schema, set_config_to_context


def _config_callback(ctx: typer.Context) -> None:
    set_config_to_context(context=ctx, settings=DataBaseSettings())


db_app = typer.Typer(short_help="Commands for simple database operations", callback=_config_callback)


@db_app.command(short_help="Create all metadata tables")
def create_all(ctx: typer.Context) -> None:
    """Create all metadata tables."""
    create_schema(config=ctx.obj)


@db_app.command(short_help="Drop all metadata tables, attributes, schema")
def drop_all(ctx: typer.Context) -> None:
    """Drop all metadata tables, attributes, schema."""
    typer.confirm("Does it really need?", abort=True)
    drop_schema(config=ctx.obj)



# flake8: noqa
from .base import BaseTable, current_session, metadata
from .statuses import DraftStatus, EmulationStatus, TestReportStatus, TestRunStatus
from .tables import (
    Draft,
    Emulation,
    EmulationRun,
    Feature,
    FeatureType,
    Scenario,
    Tags,
    TestRun,
    TestUser,
    tags_association_table,
)
from .users import GroupRole, Role, UserRole
from .utils import create_session, ensure_feature_types_exist



import typer

from overhave.base_settings import DataBaseSettings, LoggingSettings
from overhave.cli.group import overhave
from overhave.factory import ConsumerFactory
from overhave.transport import RedisStream


def _run_consumer(stream: RedisStream) -> None:
    DataBaseSettings().setup_db()
    LoggingSettings().setup_logging()
    ConsumerFactory(stream=stream).runner.run()


@overhave.command(short_help="Run Overhave Redis consumer")
def consumer(
    stream: RedisStream = typer.Option(..., "-s", "--stream", help="Redis stream, which defines application")
) -> None:
    """Run Overhave Redis consumer."""
    _run_consumer(stream)



import typer
import uvicorn

from overhave.cli.group import overhave


@overhave.command(short_help="Run Overhave API")
def api(
    port: int = typer.Option(8000, "-p", "--port", help="Service port"),
    workers: int = typer.Option(1, "-w", "--workers", help="Uvicorn workers"),
) -> None:
    """Run Overhave API."""
    from overhave.api.asgi import app
    from overhave.api.settings import OverhaveApiSettings

    settings = OverhaveApiSettings(port=port, workers=workers)
    uvicorn.run(app, **settings.dict())



# flake8: noqa
from .abstract import IFeatureExtractor
from .errors import FeatureTypeExtractionError, ScenariosTestFileNotFound
from .extractor import FeatureExtractor
from .types import FeatureTypeName
